/*
 * $Id: rb_gradient.c 354 2006-02-10 18:14:08Z tilman $
 *
 * Copyright (C) 2004 Tilman Sauerbeck (tilman at code-monkey de)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ruby.h>

#include <Evas.h>

#include "rb_evas_main.h"
#include "rb_evas.h"
#include "rb_evas_object.h"

/*
 * call-seq:
 *  Evas::Gradient.new(evas) => gradient
 *
 * Creates an Evas::Gradient object.
 */
static VALUE c_init (VALUE self, VALUE evas)
{
	CHECK_CLASS (evas, cEvas);
	GET_OBJ (evas, RbEvas, e);
	GET_OBJ (self, RbEvasObject, grad);

	grad->real = evas_object_gradient_add (e->real);

	rb_call_super (1, &evas);

	return self;
}

/*
 * call-seq:
 *  gradient.add_color(r, g, b, a, distance) => nil
 *
 * Adds a color to <i>gradient</i>.
 */
static VALUE c_add_color (VALUE self, VALUE r, VALUE g, VALUE b,
                          VALUE a, VALUE distance)
{
	GET_OBJ (self, RbEvasObject, e);

	Check_Type (r, T_FIXNUM);
	Check_Type (g, T_FIXNUM);
	Check_Type (b, T_FIXNUM);
	Check_Type (a, T_FIXNUM);
	Check_Type (distance, T_FIXNUM);

	evas_object_gradient_color_add (e->real, FIX2INT (r), FIX2INT (g),
	                                FIX2INT (b), FIX2INT (a),
	                                FIX2INT (distance));

	return Qnil;
}

/*
 * call-seq:
 *  gradient.clear_colors => nil
 *
 * Clears the colors of <i>gradient</i>.
 */
static VALUE c_clear_colors (VALUE self)
{
	GET_OBJ (self, RbEvasObject, e);

	evas_object_gradient_colors_clear (e->real);

	return Qnil;
}

/*
 * call-seq:
 *  gradient.angle => fixnum
 *
 * Returns the angle of <i>gradient</i>.
 */
static VALUE c_angle_get (VALUE self)
{
	GET_OBJ (self, RbEvasObject, e);

	return INT2FIX (evas_object_gradient_angle_get (e->real));
}

/*
 * call-seq:
 *  gradient.angle(fixnum)
 *
 * Sets the angle of <i>gradient</i>.
 */
static VALUE c_angle_set (VALUE self, VALUE val)
{
	GET_OBJ (self, RbEvasObject, e);

	Check_Type (val, T_FIXNUM);

	evas_object_gradient_angle_set (e->real, FIX2INT (val));

	return Qnil;
}

void Init_Gradient (void)
{
	VALUE c = rb_define_class_under (mEvas, "Gradient", cEvasObject);

	rb_define_method (c, "initialize", c_init, 1);
	rb_define_method (c, "add_color", c_add_color, 5);
	rb_define_method (c, "clear_colors", c_clear_colors, 0);
	rb_define_method (c, "angle", c_angle_get, 0);
	rb_define_method (c, "angle=", c_angle_set, 1);
}
